class Singleton(object):
    _instance = None
    def __new__(class_, *args, **kwargs):
        if not isinstance(class_._instance, class_):
            class_._instance = object.__new__(class_, *args, **kwargs)
        return class_._instance


import inspect

from exceptions import OverrideException


def override(func):
    def wrapper(self):
        # We need this behavior to prevent calling not overridden function without creating an object.
        try:
            getattr(super(self.__class__, self), func.__name__)()
        except Exception:
            raise OverrideException('method was not overridden!')
        return func(self)

    return wrapper


class OverrideClass:
    def __init__(self):
        source_lines = inspect.getsourcelines(self.__class__)[0]
        for i, line in enumerate(source_lines):
            line = line.strip()
            if line.split('(')[0].strip() == '@' + 'override':
                next_line = source_lines[i + 1]
                name = next_line.split('def')[1].split('(')[0].strip()
                try:
                    getattr(super(self.__class__, self), name)()
                except Exception:
                    raise OverrideException('method was not overridden!')



import inspect

from exceptions import ConstException


class MetaConst(type):
    def __getattr__(cls, key):
        return cls[key]

    def __setattr__(cls, key, value):
        raise ConstException("Const can't be changed!")


class Const(object, metaclass=MetaConst):
    def __init__(self):

        # Use global variables because if we declare it as attribute it will cause exception.
        global attrs
        global attr_name

        # Get all subclass attributes
        attrs = set(dir(self.__class__)) - set(dir(Const))

        if len(attrs) > 1:
            raise ConstException("variables with Const type must have only 1 value!")

        attr_name = ''

        # Use inspect module to get attr values
        for i in inspect.getmembers(self.__class__):
            if not i[0].startswith('_'):

                if not inspect.ismethod(i[1]):
                    attr_name = i[1]
                else:
                    raise ConstException('Const type variable cannot have any methods!')

    def __getattr__(self, name):
        return self[name]

    def __setattr__(self, name, value):
        raise ConstException("Const can't be changed!")

    def __repr__(self):
        return str(attr_name)


class ConstNameSpace(object, metaclass=MetaConst):
    def __init__(self):
        # Use global variable because if we declare it as attribute it will cause exception.
        global attrs

        # Get all subclass attributes
        attrs = set(dir(self.__class__)) - set(dir(Const))

    def __getattr__(self, name):
        return self[name]

    def __setattr__(self, name, value):
        raise ConstException("Const can't be changed!")

    def __repr__(self):
        return str(attrs)



from dataclasses import dataclass
from typing import List, Any, Union, NoReturn


class ObjectList:
    def __init__(self, value: List[object]):
        self.value: List[object] = value

    def get_object_by_spec(self, name: str, value: Any) -> Union[object, None]:
        for i in self.value:
            if getattr(i, name) == value:
                return i
        return None

    def get_all_objects_by_spec(self, name: str, value: Any) -> Union[List[object], List]:
        objects: List[object] = []
        for i in self.value:
            if getattr(i, name) == value:
                objects.append(i)
        return objects

    def add_object(self, object_to_add: object) -> NoReturn:
        self.value.append(object_to_add)

    def add_object_by_index(self, index: int, value: object) -> NoReturn:
        self.value.insert(index, value)

    def delete_object_by_index(self, index: int) -> NoReturn:
        del self.value[index]

    def delete_object(self, object_: object) -> NoReturn:
        self.value.remove(object_)

    def delete_by_object_name(self, name: str) -> NoReturn:
        for i in self.value:
            if i.__class__.__name__ == name:
                del self.value[self.value.index(i)]
                break

    def delete_all_by_object_name(self, name: str) -> NoReturn:
        for i in self.value:
            if i.__class__.__name__ == name:
                del self.value[self.value.index(i)]

    def get_by_object_name(self, name: str) -> object:
        for i in self.value:
            if i.__class__.__name__ == name:
                return i

    def get_all_by_object_name(self, name: str) -> List[object]:
        object_list: List[object] = []

        for i in self.value:
            if i.__class__.__name__ == name:
                object_list.append(i)

        return object_list

    def add_new_attr_by_index(self, index: int, name: str, value: Any) -> NoReturn:
        setattr(self.value[index].__class__, name, value)

    def __iter__(self):
        return self.value



@dataclass
class User:
    name: str
    password: str


test = ObjectList([User('Mark', '233'), User('Mak', '23ewdf3')])

print(test.get_object_by_spec('name', 'Mak'))
